<?php

namespace Drupal\media_entity_twig\Plugin\media\Source;

use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Template\TwigEnvironment;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaTypeInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Media source for Twig templates.
 *
 * @MediaSource(
 *   id = "media_entity_twig",
 *   label = @Translation("Twig Template"),
 *   description = @Translation("Provides a media entity source for rendering Twig templates."),
 *   allowed_field_types = {"twig"},
 *   default_thumbnail_filename = "generic.png",
 *   forms = {
 *     "media_library_add" = "Drupal\media_entity_twig\Form\TwigMediaLibraryAddForm",
 *   },
 * )
 */
class MediaEntityTwig extends MediaSourceBase implements ContainerFactoryPluginInterface {

  /**
   * The Twig environment service.
   *
   * Used to render Twig templates.
   *
   * @var \Drupal\Core\Template\TwigEnvironment|null
   */
  protected TwigEnvironment|NULL $twig = NULL;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   * @param array<string,mixed> $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of the plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $plugin = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    if ($plugin instanceof self) {
      $plugin->twig = $container->get('twig');
      $plugin->logger = $container->get('logger.factory')->get('media_entity_twig');
    }

    return $plugin;
  }

  /**
   * {@inheritdoc}
   *
   * @return array<string,\Drupal\Core\StringTranslation\TranslatableMarkup>
   *   An array of metadata attributes provided by the Twig media source.
   */
  public function getMetadataAttributes(): array {
    return [
      'rendered' => $this->t('Rendered Twig'),
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity.
   * @param string $attribute_name
   *   The name of the metadata attribute to retrieve.
   *
   * @return mixed
   *   The rendered Twig template as a string, or NULL if not applicable.
   */
  public function getMetadata(MediaInterface $media, $attribute_name): mixed {
    $field_name = $this->configuration['source_field'];
    $value = $media->get($field_name)->value;
    $twig = is_string($value) ? $value : '';

    switch ($attribute_name) {
      case 'rendered':
        try {
          return $this->twig?->renderInline($twig) ?? '';
        }
        catch (\Throwable $e) {
          $this->logger->error('Twig render failed: @error', ['@error' => $e->getMessage()]);
          return '';
        }
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldDefinition(MediaTypeInterface $media_type) {
    $field_name = $this->getSourceFieldName();

    $field = BaseFieldDefinition::create('twig')
      ->setLabel('Twig')
      ->setRequired(TRUE)
      ->setSettings([
        'max_length' => -1,
      ])
      ->setDisplayOptions('form', [
        'type' => 'twig',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $field->setName($field_name);

    return $field;
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldName() {
    return $this->configuration['source_field'];
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity.
   *
   * @return mixed
   *   The value of the source field.
   */
  public function getSourceFieldValue(MediaInterface $media) {
    $field_name = $this->configuration['source_field'];
    return $media->get($field_name)->value;
  }

  /**
   * {@inheritdoc}
   *
   * Prepares the view display for the media type.
   *
   * @param \Drupal\media\MediaTypeInterface $type
   *   The media type.
   * @param \Drupal\Core\Entity\Display\EntityViewDisplayInterface $display
   *   The view display to prepare.
   */
  public function prepareViewDisplay(MediaTypeInterface $type, EntityViewDisplayInterface $display): void {
    $field_definition = $this->getSourceFieldDefinition($type);

    if ($field_definition !== NULL) {
      $display->setComponent($field_definition->getName(), [
        'type' => 'twig',
        'label' => 'hidden',
      ]);
    }
  }

}
